uniform extern float4x4 WorldViewProj;

texture  g_MeshTexture : Texture;   // Color texture for mesh

float4x4 g_mWorld : World;
float4x4 g_mWorldViewProjection : WorldViewProjection; // World * View * Projection matrix

float4x4 g_mWorldInverseTranspose : WorldInverseTranspose;
float4 g_DiffuseMtrl : DiffuseMtrl;
float4 g_DiffuseLight : DiffuseLight;
float4 g_AmbientMtrl : AmbientMtrl;
float4 g_AmbientLight : AmbientLight;
float3 g_LightVecW : LightVecW;
 
float4 g_SpecularMtrl : SpecularMtrl;
float4 g_SpecularLight : SpecularLight;
float g_SpecularPower : SpecularPower;
float3 g_EyePosW : EyePosW;

float3 g_Attenuation012 : Attenuation012;

sampler MeshTextureSampler = 
sampler_state
{
    Texture = <g_MeshTexture>;    
    MipFilter = NONE
    ;
	MinFilter = NONE;
	MagFilter = NONE;
//  MipFilter = LINEAR;
//	MinFilter = LINEAR;
//	MagFilter = LINEAR;
};

void TransformVS(float3 in_pos : POSITION,
				float3 in_normal : NORMAL,
				float2 in_tex : TEXCOORD0,
				out float4 out_pos : POSITION,
				out float2 out_tex : TEXCOORD0,
 				out float4 out_pos_w : TEXCOORD1,
 				out float4 out_normal : TEXCOORD2)
{
	out_pos = mul(float4(in_pos, 1.0f), g_mWorldViewProjection);

	out_tex = in_tex;
	
	out_pos_w = mul(float4(in_pos, 1.0f), g_mWorld);
	out_normal = mul(float4(in_normal, 0.0f), g_mWorld); 
	
}

void TransformPS(float4 in_tex : TEXCOORD0,
			  float4 in_pos_w : TEXCOORD1,
			  float4 in_normal : TEXCOORD2,
			  out float4 out_color : COLOR0)
{

	float4 d_color;
	float4 s_color;
	
	d_color = float4(1,1,1,1);
	s_color = float4(0,0,0,0);
	
	float3 light_dir;
	light_dir = normalize(g_LightVecW.xyz - in_pos_w.xyz);
	
	float3 eye_dir;
	eye_dir = normalize(g_EyePosW.xyz - in_pos_w.xyz);
	
	
	// Attentuation.
	float d = distance(g_LightVecW.xyz, in_pos_w.xyz);
	float A = g_Attenuation012.x + g_Attenuation012.y*d + g_Attenuation012.z*d*d;
	
	d_color = (g_AmbientMtrl * g_AmbientLight) + (g_DiffuseMtrl * g_DiffuseLight) * max(0, dot(light_dir, in_normal.xyz)) / A;
//	d_color = saturate(d_color);

	float3  half_dir;
	half_dir = normalize(light_dir + eye_dir);
	s_color = (g_SpecularMtrl * g_SpecularLight) * pow(max(0, dot(half_dir, in_normal.xyz)), g_SpecularPower) / A;


	out_color = saturate(d_color * tex2D(MeshTextureSampler,in_tex) + s_color);

}

technique Default
{
    pass P0
    {
        // Specify the vertex and pixel shader associated with this pass.
        vertexShader = compile vs_2_0 TransformVS();
        pixelShader  = compile ps_2_0 TransformPS();

		// Specify the render/device states associated with this pass.
//		FillMode = Wireframe;
    }
}    